var activeOverlay = 0;
var added = false;

$(document).ready(function() {

  initTooltips();
  initVideoChannel();
  initServiceId();

  $('#customScale').on('keyup change', function() {
    validateRange($(this), {
      minVal: 30,
      maxVal: 200,
      invalidMsg: 'Must be from interval 30-200%.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  $('#customDuration').on('keyup change', function() {
    validateRange($(this), {
      minVal: 0,
      maxVal: 99999,
      invalidMsg: 'Must be from interval 0-99999s.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  // Fill form
  $('#customEnabled').val(serviceArrTmp[activeItem].enabled ? "Yes" : "No");
  $('#customCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });
  const resolutionId = '#customResolution';
  const videoChannelId = '#customVideoChannel';
  createResolutionList(resolutionId);
  createTriggeringElement('#customAutomationType', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule);
  onVideoChannelChange(resolutionId, videoChannelId, serviceArrTmp[activeItem].camera);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera).then(function() {
    // Callbacks for changes
    $('#mainForm .form-control').change(formChangedCallback);
    $('#saveOverlayBtn').click(saveEditedOverlayCallback);
    $('#editOverlayModalDismiss').click(cancelEditedOverlayCallback);
    $('#cancelOverlayBtn').click(cancelEditedOverlayCallback);
    $('#addOverlayBtn').click(addOverlayClickedCallback);
    $('#customImageManagerBtn').click(showImageManagerCallback);
    $('#customImgPath').on('change', function() {
      customImagePathChanged();
    });

    formChangedCallback();
  });

  showOverlayList();
  createScheduleBtn();
});

function formChangedCallback() {
  serviceArrTmp[activeItem].enabled = $('#customEnabled').val() == 'Yes' ? 1 : 0;
  serviceArrTmp[activeItem].customName = $('#customCustomName').val();

  if (!isMips()) {
    if(enabledServicesCount() > 8 || enabledServicesPerVideoChannel(serviceArrTmp[activeItem].camera) > 4) {
      serviceArrTmp[activeItem].enabled = 0;
      $('#customEnabled').val('No');
      showMessage('warning', 'A maximum of four services can be enabled simultaneously in one Video Channel and eight services can run in total.', false, 'maxFourServices');
    }
  } else {
    if(enabledServicesCount() > 2) {
      serviceArrTmp[activeItem].enabled = 0;
      $('#customEnabled').val('No');
      showMessage('warning', 'Maximally two services can be enabled simultaneously.',false, 'maxTwoServices');
    }
  }

  if (serviceArrTmp[activeItem].name == "custom") {
    serviceArrTmp[activeItem].name = "images";
  }
  let currentCameraResolutionObject = cameraResolutions[parseInt($('#customResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  serviceArrTmp[activeItem].automationType = $('#customAutomationType').val();

  serviceArrTmp[activeItem].camera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $('#customVideoChannel'));
  createCameraPreview($('#customVideoChannel'), currentCameraResolutionObject, serviceArrTmp[activeItem].camera);

  showSaveWarning();
  createServiceList(serviceArrTmp);
}

function showOverlayList() {
  var overlayListHtml = '';
  if (serviceArrTmp[activeItem].overlayList != undefined && serviceArrTmp[activeItem].overlayList.length > 0) {
    overlayListHtml += '<tbody>';
    var overlayList = serviceArrTmp[activeItem].overlayList;
    for (var i = 0; i < overlayList.length; i++) {
      overlayListHtml += '<tr>';
      var imgSrc = '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(overlayList[i].imgPath) + '&_t=' + Date.now();
      overlayListHtml += '<td style="width: 1%;"><img id="imageManagerPreview" src="' + imgSrc + '" style="max-width: 60px;" /></td>';
      overlayListHtml += '<td style="max-width: 150px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;">';
      overlayListHtml += '<span style="font-style: italic;">' + overlayList[i].imgName + '</span>';
      overlayListHtml += '</td>';

      overlayListHtml += '<td class="hidden-xs" style="text-align: center;">';
        if (overlayList[i].duration) {
          overlayListHtml += overlayList[i].duration + 's';
        } else {
          overlayListHtml += '0s';
        }
      overlayListHtml += '</td>';

      overlayListHtml += '<td class="hidden-xs" style="text-align: center;">';
        overlayListHtml += coordSystemToNiceName(overlayList[i].coordSystem) + '<br />';
        overlayListHtml += 'x: ' + overlayList[i].pos_x + ', ';
        overlayListHtml += 'y: ' + overlayList[i].pos_y;
        if (!isNaN(overlayList[i].scale)) {
          overlayListHtml += ', <br>scale: ' + convertFloatToPercentage(overlayList[i].scale) + '%';
        }
      overlayListHtml += '</td>';

      overlayListHtml += '<td style="text-align: center;">';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveUp icon formItem" style="margin-left: 0px;"><span class="fas fa-arrow-circle-up"  title="Move Up"></span></a>';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveDown icon formItem"><span class="fas fa-arrow-circle-down"  title="Move Down"></span></a>';
      overlayListHtml += '<a href="#" id="editOverlay' + i + '" class="overlayEdit formItem icon"><span class="fas fa-edit"  title="Edit"></span></a>';
      overlayListHtml += '<a href="#" id="removeOverlay' + i + '" class="overlayRemove formItem icon"><span class="fas fa-times-circle"  title="Delete"></span></a>';
      overlayListHtml += '</td></tr>';
    }
    overlayListHtml += '</tbody>';
  }
  else
  {
    serviceArrTmp[activeItem].overlayList = [];
    overlayListHtml += '<p class="text-center">No overlay set.</p>';
  }

  $('#overlayList').html(overlayListHtml);
  $('.overlayMoveUp').click(moveOverlayClicked);
  $('.overlayMoveDown').click(moveOverlayClicked);
  $('.overlayEdit').click(editOverlayClicked);
  $('.overlayRemove').click(removeOverlayClicked);

  showSaveWarning();
}

function addOverlayClickedCallback() {
  activeOverlay = serviceArrTmp[activeItem].overlayList.length;
  added = true;
  serviceArrTmp[activeItem].overlayList.push(getDefaultOverlay());
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
}

function getDefaultOverlay() {
  return {"coordSystem": "top_left", "pos_x": 10, "pos_y": 10, "imgPath": "", "imgName": "", "duration": 5, "scale": 1.0};
}

async function setEditedOverlay() {
  createPositionList('#customPosition', serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem);
  $('#customPosX').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x);
  $('#customPosY').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y);
  let customScale = convertFloatToPercentage(serviceArrTmp[activeItem].overlayList[activeOverlay].scale);
  if (isNaN(customScale)) {
    customScale = 100;
  }
  $('#customScale').val(customScale);
  $('#customDuration').val(serviceArrTmp[activeItem].overlayList[activeOverlay].duration);

  // Create image list field
  await createImageList('#customImgPath', true, serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath);
  customImagePathChanged();
}

function customImagePathChanged() {
  var options = $('#customImgPath option');
  var selectedIndex = $('#customImgPath')[0].selectedIndex;
  if (options.length == 0) {
    $('#customPreviewEditOverlay').html('');
    return;
  }
  var fileSrc = options[selectedIndex].getAttribute('value');
  $('#imagePreviewEditOverlay').html('<img id="imageManagerPreview" src="/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(fileSrc) + '" />');
}

async function saveEditedOverlayCallback() {
  added = false;
  let imageList = await imageManager.getFileList();
  var image = findImageByPath(imageList, $('#customImgPath').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem = $('#customPosition').val();
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x = parseInt($('#customPosX').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y = parseInt($('#customPosY').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath = image.path;
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgName = image.name;
  serviceArrTmp[activeItem].overlayList[activeOverlay].duration = parseInt($('#customDuration').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].scale = convertPercentageToFloat($('#customScale').val(), 100);

  showOverlayList();
}

function cancelEditedOverlayCallback() {
  if (added) {
    added = false;
    serviceArrTmp[activeItem].overlayList.pop();
  } else {
    serviceArrTmp[activeItem].overlayList[activeOverlay] = savedOverlay;
  }
  showOverlayList();
}

function moveOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var linkElement = $(targ).closest('a');
  var index = linkElement.attr('id').substring(11);
  var newIndex = parseInt(index) + 1;
  if (linkElement.hasClass('overlayMoveUp'))
    newIndex = parseInt(index) - 1;
  if (newIndex < 0 || newIndex >= serviceArrTmp[activeItem].overlayList.length)
    return false;
  var tmp = serviceArrTmp[activeItem].overlayList[index];
  serviceArrTmp[activeItem].overlayList[index] = serviceArrTmp[activeItem].overlayList[newIndex];
  serviceArrTmp[activeItem].overlayList[newIndex] = tmp;
  showOverlayList();
  return false;
}

function editOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(11);
  activeOverlay = index;
  savedOverlay = serviceArrTmp[activeItem].overlayList[activeOverlay];
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
  return false;
}

function removeOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(13);
  serviceArrTmp[activeItem].overlayList.splice(index, 1);
  showOverlayList();
  return false;
}