function PtzManager() {

  let getList = async function(cameraChannel) {
    let result = await getData({
      'query': 'presetposcam',
      'camera': ++cameraChannel,
    });

    const posNameList = result.split('\n');
    const ptzList = [];
    for (var i = 1; i < posNameList.length - 1; i++) {
      var presetName = posNameList[i].split('=');
      ptzList.push(presetName[1].trim());
    }

    return ptzList;
  }

  let getListData = async function(cameraChannel) {
    let result = await getData({
      'query': 'presetposcamdata',
      'camera': ++cameraChannel,
    });

    result = result.split('\n');
    result.shift();
    result.pop();

    resultArr = [];
    result.forEach((position, index) => {
      position = position.split(':');
      let name = position[0].split('=');
      let pan = position[1].split('=');
      let tilt = position[2].split('=');
      let zoom = position[3].split('=');

      resultArr[index] = {
        name: name[1],
        pan: parseFloat(pan[1]).toFixed(2),
        tilt: parseFloat(tilt[1]).toFixed(2),
        zoom: parseInt(zoom[1]).toString(),
      };
    });

    return resultArr;
  }

  let getData = async function(params) {

    let ptz_cgi_url = createUrlWithParams(params);

    const result = await $.get(ptz_cgi_url).catch((jqXHR, textStatus, errorThrown) => {
      throw new Error(errorThrown || 'Network error');
    });

    handleError(result);

    return result;
  }

  let getPosition = async function() {

    let result = await getData({
      'query': 'position',
    });

    return parseParams(result);
  }

  let getCurrentPosition = async function () {
    let positionData = await getListData();
    let current = await getPosition();
    let result = '';
    positionData.forEach(item => {
      if (item.pan === current.pan && item.tilt === current.tilt && item.zoom === current.zoom) {
        result = item.name;
      }
    });
    return result;
  }

  let setPosition = async function(presetName, cameraChannel) {
    await getData({
      'gotoserverpresetname': presetName.trim(),
      'camera': ++cameraChannel,
    });
  }

  let createUrlWithParams = function(params) {
    let ptz_cgi_url = new URL('/axis-cgi/com/ptz.cgi', window.location.origin);
    let urlParams = new URLSearchParams(ptz_cgi_url.search);
    for (let key in params) {
      urlParams.set(key, params[key]);
    }
    ptz_cgi_url.search = urlParams.toString();
    return ptz_cgi_url;
  }

  let handleError = function(result) {
    const err = /^Error:/.test(result);
    if (err) {
      throw new Error(result);
    }
  }

  return {
    getList,
    getCurrentPosition,
    getPosition,
    setPosition
  }

}