
var alertID = 0;
var propertiesResponse = null;

function getImgSize(imgSrc) {
  var promise = new Promise(function(resolve, reject) {
    var newImg = new Image();

    newImg.onload = function() {
      resolve({width: newImg.width, height: newImg.height});
    }

    newImg.src = imgSrc; // this must be done AFTER setting onload
  });
  return promise;
}

function firstToUpperCase(str) {
  return str.substr(0, 1).toUpperCase() + str.substr(1);
}

async function getParameter(paramList) {
  try {
    const result = await $.get("/axis-cgi/param.cgi?action=list&group=" + paramList);
    return parseParams(result);
  } catch (e) {
    throw new Error("Can't get parameters");
  }
}

function getIOPortParameter(paramList) {
  var promise = new Promise(function(resolve, reject) {
    $.get("/axis-cgi/io/port.cgi?" + paramList, function(data) {
      resolve(parseParams(data));
    });
  });
  return promise;
}

function parseParams(data) {
  var params = {};
  var lines = data.split('\n');
  for (var i = 0; i < lines.length; i++) {
    var pos = lines[i].indexOf('=');
    if (pos != -1) {
      var name = lines[i].substring(0, pos).trim().toLowerCase();
      var value = lines[i].substring(pos + 1).trim();
      if (name.length && value.length) {
        params[name] = value;
      }
    }
  }
  return params;
}

function getServices() {
  var promise = new Promise(function(resolve, reject) {
    $.ajax({
      method: "GET",
      cache: true,
      url: "/local/camoverlay/api/services.cgi?action=get"
    }).done(function(data) {
      if (data.services) {
        resolve(data.services);
      } else {
        reject();
      }
    }).fail(function() {
      reject();
    });
  });
  return promise;
}

function setServices(services) {
  var promise = new Promise(function(resolve, reject) {
    $.ajax({
      method: "POST",
      cache: true,
      url: "/local/camoverlay/api/services.cgi?action=set",
      data: services
    }).done(function(msg) {
      if (msg.status == 200) {
        resolve();
      } else {
        console.error(msg);
        reject();
      }
    }).fail(function() {
      reject();
    });
  });
  return promise;
}

function showSaveWarning() {
  if (JSON.stringify(serviceArr[activeItem]) !== JSON.stringify(serviceArrTmp[activeItem])) {
    const warningAlertName = 'saveWarning';
    const warningText = 'Unsaved changes';
    let html = '<div id="' + warningAlertName + '" class="alert alert-warning d-inline-flex" title="' + warningText + '">';
    html += '<strong><span class="fas fa-exclamation-triangle"></span></strong><span class="ml-2 d-none d-sm-block">' + warningText + '</span>';
    html += '</div>';
    $('#warningArea').html(html);
    if ($(window).width() < 769) { // Display tooltip only on mobile
      $('#'+warningAlertName).tooltip();
    }
  } else {
    hideSaveWarning();
  }
}

function validate(element, invalidValue, invalidMsg){
  var validation = (element.val() === invalidValue) ? 'is-invalid' : '';
  var invalidDiv = '<div class="invalid-feedback">' + invalidMsg + '</div>';

  if(validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
  }
}

function disableSaveBtn(){
  $('#saveBtn').prop('disabled', true);
}

function enableSaveBtn(){
  $('#saveBtn').prop('disabled', false);
}

function configExport() {
  $('.downloadConfiguration').off('click').on('click', function (e) {
    e.preventDefault();
    var data = "text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(serviceArrTmp[activeItem]));
    var a = document.createElement('a');
    a.href = 'data:' + data;
    a.download = 'exportConfiguration-' + new Date().getTime().toString() + '.json';
    a.click();
  });
}

function configImport() {
  $('.uploadConfiguration').off('click').on('click', function() {
    $('#configurationFileUpload').trigger('click');
  });

  $('#configurationFileUpload').on('change', function() {
    var files = document.getElementById('configurationFileUpload').files;
    if (files.length <= 0) {
      showMessage('warning', 'Empty file input.', false, 'emptyFileInput');
    }
    var fr = new FileReader();
    fr.onload = function(e) {
      try {
        var importedData = JSON.parse(e.target.result);
        importedData.id = serviceArrTmp[activeItem].id;
        serviceArrTmp[activeItem] = importedData;
        $('[data-toggle="tooltip"], .tooltip').tooltip('dispose'); //destroy tooltips from previous page
        createServiceList(serviceArrTmp);
        setServiceItemActive(activeItem);
        showMessage('success', 'Configuration was successfully imported', false, 'configurationUploadSuccess');
      } catch(e) {
        showMessage('danger', 'JSON parsing goes wrong.', false, 'jsonParseError')
      }
    };
    fr.readAsText(files.item(0));
    return false;
  });
}

$.fn.showSpinner = function(text = '') {
  var loadingDiv = '<div class="loadingDiv align-items-center align-items-center text-center row rounded h-100" style="position:absolute;top:0;bottom:0;left:15px;right:15px;background-color: #f7f7f7;z-index: 9999;">';
  loadingDiv += '<div class="col"><p class="text-primary fas fa-2x fa-spinner fa-pulse"></p>';
  if (text !== '') loadingDiv += '<p class="text-muted mb-0">' + text + '</p>';
  loadingDiv += '</div></div>';
  $(this).append(loadingDiv);
};

$.fn.hideSpinner = function() {
  $(this).find('.loadingDiv').remove();
};

function hideSaveWarning() {
  $('#warningArea').html('');
}

function createResolutionList(selectID) {
  var selected = 0;
  var cameraResolution = -1;
  $(selectID).empty();
  var resItem = $(selectID)[0];
  for (let i = 0; i < cameraResolutions.length; i++) {
    if (cameraResolutions[i].width === serviceArrTmp[activeItem].width && cameraResolutions[i].height === serviceArrTmp[activeItem].height) {
      selected = i;
    }
    if (cameraResolutions[i].width === resolution.width && cameraResolutions[i].height === resolution.height) {
      cameraResolution = i;
    }
    resItem.options[resItem.options.length] = new Option(cameraResolutions[i].width + ' x ' + cameraResolutions[i].height, i);
  }
  resItem.selectedIndex = selected;
  if (cameraResolution != -1) {
    $(selectID + ' option[value="' + cameraResolution + '"]').css('background-color', '#B6DAFF');
  }
}

function createTriggeringElement(selectId, selectedVal, scheduleVal) {
  const element = $(selectId);

  // Change event
  element.change(function () {
    checkHwIoState(element);
    toggleScheduleBtn(element);
  });

  // Check if schedule is empty - set manual triggering, otherwise set schedule triggering
  if(selectedVal === undefined) {
    selectedVal = "schedule";
    if(scheduleVal === "") {
      selectedVal = "manual";
    }
  }

  getParameter('Input.NbrOfInputs').then(function (responseData) {
    const nbrOfInputs = parseInt(responseData['input.nbrofinputs']);
    if (nbrOfInputs === 0) {
      return;
    }
    let paramNames=[];
    for (let i = 0; i < nbrOfInputs; i++) {
      paramNames.push('ioport.i'+i+'.direction');
      paramNames.push('ioport.i'+i+'.input.name');
    }
    getParameter(paramNames).then(function (retrievedParameters) {
      for (let i = 0; i < nbrOfInputs; i++) {
        if(retrievedParameters['ioport.i'+i+'.direction'] === 'input') {
          let input_name = retrievedParameters['ioport.i'+i+'.input.name'];
          element.append(new Option('I/O port (' + input_name + ')', 'input'+i));
        }
      }
      if (element.find('option[value=' + selectedVal + ']').length === 1) {
        element.val(selectedVal);
      } else {
        showMessage('warning', 'Trigger option' + (selectedVal === undefined ? ' ' : selectedVal) + 'is not valid. Reseting to default trigger.', false, 'triggerValueNotFound');
        element.val('manual');
      }
      element.change();
    });
  });
}

function toggleScheduleBtn(triggeringElement) {
  let isSchedule = triggeringElement.val().match(/schedule/) !== null;
  $('#scheduleBtn').toggle(isSchedule);
}

function checkHwIoState(triggeringElement) {
  let isHwInput = triggeringElement.val().match(/input\d+/) !== null;

  if (typeof timeOut === 'number') {
    clearTimeout(timeOut);
  }

  if (isHwInput) {
    let inputNumber = parseInt(triggeringElement.val().replace('input', '')) + 1;
    getIOPortParameter("checkactive="+inputNumber).then(function (responseData) {
      if(responseData['port' + inputNumber] === "active") {
        $('.hw_badge.hw_badge_off').hide();
        $('.hw_badge.hw_badge_on').show();
      } else {
        $('.hw_badge.hw_badge_on').hide();
        $('.hw_badge.hw_badge_off').show();
      }
      timeOut = setTimeout(checkHwIoState, 5000, triggeringElement);
    });
  } else {
    $('.hw_badge').hide();
  }
}

/**
 * If artpec 5 hide the video channel
 */
function initVideoChannel() {
  if (isMips()) {
    $('select[id*="VideoChannel"]').closest('div.input-group').hide();
  }
}

async function createVideoChannelElement(selectID, selectedVal) {
  const paramResponse = await getParameter('Image.NbrOfConfigs');
  const configCount = parseInt(paramResponse['image.nbrofconfigs']);

  let imageParams = 'Properties.Image.Resolution';
  for (var i = 0; i < configCount; i++) {
    imageParams += ',Properties.Image.I' + i + '.Resolution,Image.I' + i + '.Appearance.Resolution,Image.I' + i + '.Name,Image.I' + i + '.Source,Image.I' + i + '.Enabled';
  }

  const propertiesResponse = await getParameter(imageParams);
  const selectedVideoChannel = createVideoChannelList(selectID, selectedVal, configCount, propertiesResponse); // Return selected view area number

  if (selectedVideoChannel < 0) {
    $(selectID + ' option[value="-1"]'); // selected view are is not existing
  } else {
    $(selectID + ' option[value="' + selectedVideoChannel + '"]').prop('selected', true);
  }
  $(selectID).change();
}

function saveVideoChannelValue(param, element) {
  var value = parseInt(element.val());
  if(value > -1) { // not assing: -1
    param = value;
  } else {
    param = -1
  }
  return param
}

function onCustomNameKeyUp(e) {
  let value = $(e.target).val();
  $('.serviceListItemActive > .serviceName').html('<span>' + escapeHtml(value) + '</span>');
  showSaveWarning();
}

/**
 *
 * @param {string} resolutionId - id of select element for resolution
 * @param {string} videoChannelId - id of select element for video channel
 * @param {number} value - value of video channel
 */
function onVideoChannelChange(resolutionId, videoChannelId, value) {
  $(videoChannelId).change(function() {
    let selectedCamera = saveVideoChannelValue(value, $(this));
    selectedCamera = selectedCamera < 0 ? 0 : selectedCamera;
    cameraResolutions = createNewResolutionList(params['properties.image.i' + selectedCamera + '.resolution'] || params['properties.image.resolution']);
    resolution = createResolutionObject(params['image.i' + selectedCamera  + '.appearance.resolution']);
    createResolutionList(resolutionId);
    formChangedCallback();
  });
}

/**
 * @param {jQuery} element - jquery element object of videoChannel
 * @param {Object} resolution - contains width and height prop
 * @param {number} videoChannel - number of channel of camera
 */
function createCameraPreview(element, resolution, videoChannel) {
  videoChannel = parseInt(element.val()) < 0 ? null : videoChannel;
  showImageFromCamera('imagePreviewCamera', resolution, videoChannel);
}

function createVideoChannelList(selectID, selectedVal, videoChannelCount, cameraProperties) {
  var selected = -1;
  var element = $(selectID);
  element.empty();
  var resItem = element[0];
  resItem.options[resItem.options.length] = new Option('Not assign', -1);
  for (var i = 0; i < videoChannelCount; i++) {
    if (cameraProperties['image.i' + i + '.enabled'] === 'yes' || i === 0) {
      if (i === selectedVal) {
        selected = i;
      }
      resItem.options[resItem.options.length] = new Option(cameraProperties['image.i' + i + '.name'], i);
    }
  }
  return selected;
}

/**
 * Help to set default resolution for video channel
 * @param {string} resolutionProperty
 * @returns {{width: number, height: number}}
 */
function createResolutionObject(resolutionProperty) {
  const res = resolutionProperty.split('x');
  if (res.length > 1) {
    return {
      width: parseInt(res[0].trim()),
      height: parseInt(res[1].trim())
    }
  }
}

/**
 * Help to create new resolution
 * @param {string} resolutionProperties
 * @returns {this}
 */
function createNewResolutionList(resolutionProperties) {
  let resolutionList = getAvailableResolution(resolutionProperties);
  return resolutionList.sort(sortFunc);
}

function getAvailableResolution(params) {
  let cameraResolutions = [];
  let resolutionsStr = params.split(',');
  for (let i = 0; i < resolutionsStr.length; i++) {
    let resObject = createResolutionObject(resolutionsStr[i]);
    cameraResolutions.push(resObject);
  }
  return cameraResolutions;
}

function createPositionList(selectID, selectedVal) {
  $(selectID).html('');
  var posItem = $(selectID);
  posItem.empty();
  posItem.append(new Option('Top Left', 'top_left'));
  posItem.append(new Option('Top Center', 'top'));
  posItem.append(new Option('Top Right', 'top_right'));
  posItem.append(new Option('Left Center', 'left'));
  posItem.append(new Option('Center', 'center'));
  posItem.append(new Option('Right Center', 'right'));
  posItem.append(new Option('Bottom Left', 'bottom_left'));
  posItem.append(new Option('Bottom Center', 'bottom'));
  posItem.append(new Option('Bottom Right', 'bottom_right'));
  posItem.val(selectedVal);
}

function coordSystemToNiceName(coordSystem) {
  if (coordSystem == 'top_left')
    return 'Top Left';
  else if (coordSystem == 'top')
    return 'Top Center';
  else if (coordSystem == 'top_right')
    return 'Top Right';
  else if (coordSystem == 'left')
    return 'Left Center';
  else if (coordSystem == 'center')
    return 'Center';
  else if (coordSystem == 'right')
    return 'Right Center';
  else if (coordSystem == 'bottom_left')
    return 'Bottom Left';
  else if (coordSystem == 'bottom')
    return 'Bottom Center';
  else if (coordSystem == 'bottom_right')
    return 'Bottom Right';
}

/**
 * Attach validation to elements in form
 * @param options - configuration array
 */
$.fn.attachValidation = function(options) {
  const element = $(this); //input element

  validatePattern(element, options); //validate on start

  element.on('keyup', function() { //validation on keyup event
    validatePattern(element, options);
  });
};

/**
 * Validation with pattern
 * @param {jquery} elem - jQuery wrapper $(this)
 * @param {object} options - configuration array
 */
function validatePattern(elem, options) {
  const element = elem;
  let pattern = options.pattern || element.attr('pattern');
  let regex = RegExp(pattern);
  let pattern_msg = options.invalidMsg || element.attr('data-pattern-title');
  let valid = regex.test(element.val());
  showValidationResult(element, valid, pattern_msg, options.disableBtnId);
}

/** Validate element with range
 *
 * @param {jquery} jqueryElement - jQuery wrapper $(this)
 * @param {object} options - configuration array
 */
function validateRange(jqueryElement, options) {
  const elementToValidate = jqueryElement;
  const value = parseInt(elementToValidate.val());
  let validResult = false;
  if (!isNaN(value) && options.minVal <= value && value <= options.maxVal) {
    validResult = true;
  }
  showValidationResult(elementToValidate, validResult, options.invalidMsg, options.disableBtnId);
}

/** Show validation result
 *
 * @param {jquery} element - jQuery wrapper $(this)
 * @param {boolean} valid - regex result if is valid
 * @param {string} invalidMsg - error msg displayed in body
 * @param {string} disableBtnId - id of btn to disable
 */
function showValidationResult(element, valid, invalidMsg, disableBtnId){
  let validation = valid ? '' : 'is-invalid';
  const invalidDiv = '<div class="invalid-feedback text-right mr-3">' + invalidMsg + '</div>';

  if(validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
    element.closest('.input-group').find('.invalid-feedback').show();
  }
  const element_button_to_disable = $('#' + disableBtnId);
  const invalid_elements_in_form = element.closest('form').find('.is-invalid').length > 0;
  element_button_to_disable.prop('disabled', invalid_elements_in_form);
}

/** Replace special chars - user input sanitization
 * @param {string} string - string is sanitized
 */
function replaceSpecialChars(string) {
  const map = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    "~": '&#126;',
    '"': '&quot;',
    "'": '&#x27;',
    '`': '&#96;',
    '\\': '&#92;',
    "/": '&#x2F;',
    "(": '&#40;',
    ")": '&#41;',
    ";": '&#59;',
    ".": '&#46;',
    "$": '&#36;',
    "%": '&#37;',
    "#": '&#35;',
    "!": '&#33;',
    "?": '&#63;',
    "|": '&#124;',
    "{": '&#123;',
    "}": '&#125;',
    "+": '&#43;',
  };
  return string.replace(/[&<>~"'`\\/();.$%#!?|{}+]/ig, (match) => (map[match]));
}

function initServiceId() {
  $('#serviceID').text("Service ID: " + serviceArrTmp[activeItem].id);
}

/**
 * Create font list options and set value
 * @param {string} jquerySelector
 * @param {string} value
 * @returns {Promise<void>}
 */
async function createFontList(jquerySelector, value) {
  await createFontListOptions(jquerySelector);
  let existingOption = $(jquerySelector + ' option[value="'+ value + '"]');
  if (value === "" || existingOption.length === 0) {
    value = "classic";
  }
  $(jquerySelector + ' option[value="' + value + '"]').attr('selected', 'selected');
  $(jquerySelector).change();
}

/**
 * Create HTML options for select
 * @param {string} jquerySelector
 * @returns {Promise<void>}
 */
async function createFontListOptions(jquerySelector) {
  let fontList = await fontManager.getFileList();
  $(jquerySelector)
    .empty()
    .append(new Option('Classic text', 'classic', false, false))
    .append(new Option('Digital text', 'digital', false, false));

  fontList.forEach(item => {
    $(jquerySelector).append(new Option(item.name, item.path.replace('file://', ''), false, false));
  });
}

/**
 * Create image list options and set value
 * @param {string} jquerySelector
 * @param {boolean} allowUrl
 * @param {string} value
 * @returns {Promise<void>}
 */
async function createImageList(jquerySelector, allowUrl, value) {
  await createImageListInOption(jquerySelector, allowUrl);
  if (value === '') { // Not choose image
    $(jquerySelector + ' option:first').attr('selected', 'selected');
  } else {
    $(jquerySelector + ' option[value="' + value + '"]').attr('selected', 'selected');
  }
  $(jquerySelector).change();
}

/**
 * Create HTML options for select
 * @param {string} jquerySelector
 * @param {boolean} allowUrl
 * @returns {Promise<void>}
 */
async function createImageListInOption(jquerySelector, allowUrl) {
  let imageList = await imageManager.getFileList();
  const allowGif = !isMips();
  $(jquerySelector).empty();

  imageList.forEach(item => {
    let path = item.path;
    let isUrl = (item.storage === "url");
    let isGif = path.includes('.gif');
    if (!allowUrl) {
      //in customGraphics, controlRoom is not valid whole url, only without file://
      path = path.replace('file://', '');
    }
    if ((allowUrl || !isUrl) && (allowGif || !isGif)) {
      $(jquerySelector).append(new Option(item.name, path, false, false));
    }
  });
}

function findImageByPath(imageList, imagePath) {
  for (var i = 0; i < imageList.length; i++) {
    if (imageList[i].path === imagePath) {
      return imageList[i];
    }
  }
  return null;
}

function parseBool(str) {
  if (str.length === null) {
    return str === 1;
  } else {
    return str === "true";
  }
}

function hexToRgb(hex){
  var c;
  if(/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)){
    c= hex.substring(1).split('');
    if(c.length== 3){
      c= [c[0], c[0], c[1], c[1], c[2], c[2]];
    }
    c= '0x'+c.join('');
    return [(c>>16)&255, (c>>8)&255, c&255].join(',')+',1';
  }else{
    throw new Error('bad HEX input');
  }
}

function rgbToHex(rgb){
  rgb = rgb.match(/^[\s+]?[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i);
  if(rgb && rgb.length === 4){
    var hex =  "#" + ("0" + parseInt(rgb[1],10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[2],10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[3],10).toString(16)).slice(-2);
    return hex;
  }else{
    throw new Error('bad RGBA input');
  }
}

function escapeHtml(html) {
  return html.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
}

function convertFloatToPercentage(value) {
  return Math.round(value * 100);
}

function convertPercentageToFloat(value, defaultValue) {
  if (value === "") {
    value = defaultValue;
  }
  return parseFloat(value) / 100;
}