var app_name = 'CamOverlay';
const appNameLowered = app_name.toLowerCase();

var appEnabled = 0;
var serviceArr = [];
var serviceArrTmp = [];

var activeItem = -1;
var itemForRemove = 0;

var alertID = 0;
var cameraPreviewUrlPath = '/axis-cgi/mjpg/video.cgi';

var cameraResolutions = [];
var resolution = {};

var scheduleTmpArr = [];

var installPage = 'installApp.html';
var defaultPage = 'settings.html';

var params;
var appVersion;
var serverUrl;
var serialNumber;
var cameraModel;
var cameraFirmware;
var cameraArchitecture;

var licenseActivated = -1;
var licenseExpire = -1;
var licenseCode = -1;

var previewRequests = [];

var is_ie = /MSIE|Trident/.test(window.navigator.userAgent);

let imageManager = ImageManager(appNameLowered);
let fontManager = FontManager(appNameLowered);
let ptzManager = PtzManager();

$(document).ready(async function() {

  $("#addServiceModal .col-md-5 img").mouseover(function() {
      $hover = $(this).next().attr("src");
      $(this).attr('src', $hover);
  }).mouseout(function() {
      $original = $(this).attr("original");
      $(this).attr('src', $original);
  });

  $.ajaxSetup ({
    // Disable caching of AJAX responses
    cache: false
  });

  let paramNameList = await getParamNameList();

  params = await getParameter(paramNameList);
  serverUrl = params['camoverlay.serverurl'];
  appEnabled = parseInt(params['camoverlay.enabled']);
  appVersion = params['camoverlay.version'];
  serialNumber = params['properties.system.serialnumber'];
  cameraModel = params['brand.prodnbr'];
  cameraFirmware = params['properties.firmware.version'];
  cameraArchitecture = params['properties.system.architecture'];

  // Create top application menu
  renderMenu('', serverUrl);
  $('.addServiceBtn').mouseenter(function(){
    var secondImgSrc = $(this).find('.secondImg').attr('src');
    $(this).find('.firstImg').attr('src',secondImgSrc);
  });
  $('.addServiceBtn').mouseleave(function(){
    var firstImgSrc = $(this).find('.firstImg').attr('original');
    $(this).find('.firstImg').attr('src',firstImgSrc);
  });

  // Replace server links
  replaceServerUrl(serverUrl);

  // Show CamOverlay version
  $('#versionNumber').html(appVersion);

  // Start acap application
  await startAcapApp();

  // Generate service list component
  try {
    serviceArr = await getServices();
    serviceArrTmp = JSON.parse(JSON.stringify(serviceArr)); // copy array
  } catch(e) {
    showMessage('warning', 'Parse settings failed.');
    serviceArr = [];
    serviceArrTmp = [];
  }

  createServiceList(serviceArr);
  setServiceItemActive(0);

  if (serviceArrTmp[activeItem] !== undefined) { // If empty config array
    let selectedCamera = serviceArrTmp[activeItem].camera < 0 ? 0 : serviceArrTmp[activeItem].camera;

    // Available resolutions for all video channels
    cameraResolutions = createNewResolutionList(params['properties.image.i' + selectedCamera + '.resolution'] || params['properties.image.resolution']);

    // Default resolution for all video channels
    resolution = createResolutionObject(params['image.i' + selectedCamera + '.appearance.resolution']);
  }

  // Disable text overlay
  $.get('/axis-cgi/param.cgi?action=update&Image.I0.Text.TextEnabled=no');

  // Set secondary DNS server
  var secondaryDNS = params['network.dnsserver2'];
  if (secondaryDNS.indexOf('0.0.0.0') != -1) {
    $.get('/axis-cgi/param.cgi?action=update&Network.DNSServer2=8.8.8.8');
  }

  // CamOverlay is suported from fw 7.10 or 9.10 on armv7hf
  var fwSupported = [7, 10];
  if (!isMips()) fwSupported = [9, 10];
  var fw = cameraFirmware.split('.');
  for (var i = 0; i < fwSupported.length && i < fw.length; i++) {
    if (parseInt(fw[i]) < fwSupported[i]) {
      showMessage('danger', 'CamOverlay is compatible with camera firmware ' + fwSupported.join('.') + ' or later. Your camera is using ' + cameraFirmware + ' now. Please check you have the latest firmware on your camera.', true);
      break;
    } else if (parseInt(fw[i]) > fwSupported[i]) {
      break;
    }
  }

  // Check license
  validateLicense(serverUrl, serialNumber, cameraModel, cameraFirmware, appVersion).then(function(response) {
    if (response == undefined) {
      licenseVerificationFailed();
    } else {
      if (response.code == 200) {
        var resData = response.data;

        // Save license to camera
        setParameter('camoverlay', 'Signature', resData.Signature).then(function() {
          setParameter('camoverlay','Expire', resData.Expire).then(function() {});
        });

        licenseActivated = resData.Activated;
        licenseExpire = resData.Expire;
        licenseCode = resData.LicenseCode;

        // Check camera time
        setTimeout(function() {   // Wait for acap start
          $.get('/local/camoverlay/api/camera_time.cgi', function(timeResponse) {
            if (!timeResponse.state) {
              showMessage('warning', 'Camera time error: ' + timeResponse.message + ' <a href="#" class="btn btn-light" onclick="window.open(\'/#settings/system/localization/dateTime\', \'_blank\')">Date & Time</a>', true);
            }
          }).fail(function(err) {
            console.log(err);
            showMessage('warning', 'Could not validate camera time. <a href="#" class="btn btn-light" onclick="window.open(\'/#settings/system/localization/dateTime\', \'_blank\')">Date & Time</a>', true);
          });
        }, 10000);

        // Check trial
        var expire = parseInt(resData.Expire);
        if (expire != 0) {
          var d = new Date(expire * 1000)
          var msg = 'Camera has a trial license which expires on ' + d.getDate() + '.' + (d.getMonth() + 1) + '.' + d.getFullYear() + '.';
          msg += ' <button class="btn btn-light" onclick="showLicenseModal(true, true)">Upgrade</button>';
          showMessage('warning', msg, true);
        }

        // Check latest version
        var lastVersion = resData.LatestACAPVersion.split('.');
        //var lastVersion = ['1', '0', '6'];
        var currentVersionParam = params['camoverlay.version'];
        var currentVersion = currentVersionParam.split('.');

        for (var i = 0; i < lastVersion.length && i < currentVersion.length; i++) {
          if (parseInt(currentVersion[i]) < parseInt(lastVersion[i])) {
              showMessage('info', 'New ' + app_name + ' App ' + resData.LatestACAPVersion + ' is available. <a href="#" class="btn btn-light" onclick="prepareUpdate(serverUrl, \'' + resData.LatestACAPVersion + '\', \'' + currentVersionParam + '\')">Update now</a>', true);
            break;
          }
          else if (parseInt(currentVersion[i]) > parseInt(lastVersion[i]))
            break;
        }


        // Show add service window
        if (serviceArr.length == 0 && !is_ie) {
          $("#addServiceModal").modal({show: true});
        }
      } else if (response.code == 489) { // Camera is not licensed
        showLicenseModal(false, true);
        showExpiredLicense();
        setParameter('camoverlay', 'Signature', '');
      } else {
        licenseVerificationFailed();
      }
    }

    // When CO is turned on/off via API, UI must reflect and change global param 'enabled'
    if (appEnabled === 0) {
      turnOffAllServices();
    }

    $(".addServiceBtn").click(serviceAddClickedCallback);
    $("#removeServiceBtn").click(serviceRemoveClickedCallback);
    $("#saveBtn").click(saveBtnCallback);
    $("#discardOkBtn").click(discardBtnCallback);
    $('#scheduleAddBtn').click(scheduleAddClickedCallback);
    $('#scheduleSaveBtn').click(scheduleSaveClickedCallback);
    $('#licenseInfoBtn').click(showLicenseInfoCallback);
    $('#imageManagerBtn').click(showImageManagerCallback);
    $('#fontManagerBtn').click(showFontManagerCallback);
    $('#controlRoomBtn').click(showControlRoomCallback);
  });
});

async function getParamNameList() {
  const result = await getParameter('Image.NbrOfConfigs');
  const nbrOfConfigs = parseInt(result['image.nbrofconfigs']);

  // General params
  let paramsList = [
    app_name + '.ServerUrl',
    app_name + '.Enabled',
    app_name + '.Version',
    'Properties.System.SerialNumber',
    'Properties.System.Architecture',
    'Properties.Firmware.Version',
    'Properties.Image.Resolution',
    'Image.I0.Appearance.Resolution',
    'Network.DNSServer2',
    'Brand.ProdNbr',
  ];

  // Add image params
  for (let i = 0; i < nbrOfConfigs; i++) {
    let imageParams = [
      'Properties.Image.I' + i + '.Resolution',
      'Image.I' + i + '.Appearance.Resolution',
      'Image.I' + i + '.Name',
      'Image.I' + i + '.Source',
      'Image.I' + i + '.Enabled',
    ]
    paramsList = paramsList.concat(imageParams);
  }
  return paramsList.join(",");
}

function turnOffAllServices() {
  for (var i = 0; i < serviceArrTmp.length; i++) {
    serviceArrTmp[i].enabled = 0;
  }
  saveBtnCallback(true);
}

function licenseVerificationFailed() {
  getParameter('camoverlay.signature').then(function(param) {
    if (!param.hasOwnProperty('camoverlay.signature') || param['camoverlay.signature'].length == 0) {
      showLicenseModal(false, false);
    } else {
      showMessage('warning', 'License cannot be verified.');
      // Show add service window
      if (serviceArr.length == 0 && !is_ie) {
        $("#addServiceModal").modal({show: true});
      }
    }
  });
}

function sortFunc(a, b) {
  return b.width - a.width;
}

function nameHelper(service, alternativeName) {
  let name = alternativeName;
  let alternativeNameWithoutHtmlTags = alternativeName.replace(/(<([^>]+)>)/ig," ");
  if (service['customName']) {
    name = '<span class="no-tooltip" title="' + alternativeNameWithoutHtmlTags + '">' + escapeHtml(service['customName']) + '</span>';
  }
  return name;
}

function createServiceAddHtml() {
  return '<div id="serviceListItemAdd" class="serviceListItem bg-dark" data-toggle="modal" data-target="#addServiceModal">' +
    '<div style="position: absolute; top: 50%; left: 50%; transform: translateY(-50%) translateX(-50%);">' +
    '<span class="fas fa-2x fa-plus-circle"></span>' +
    '</div>' +
    '</div>';
}

function createServiceCardHTML(serviceItem, index) {
  const serviceName = {
    accuweather: 'Weather<br /><span class="small" style="font-weight: normal;">(AccuWeather)</span>',
    custom: 'Images',
    images: 'Images',
    ptz: 'PTZ<br />Overlay',
    ptzCompass: 'PTZ Compass<br />',
    infoticker: 'Info<br />Ticker',
    instant: 'Instant<br />Overlay',
    sportzcast: 'Scoreboard<br /><span class="small" style="font-weight: normal;">(Sportzcast)</span>',
    textAndBackground: 'Custom<br>Graphics',
    customGraphics: 'Custom<br>Graphics',
    pip: 'Picture In<br />Picture',
    scripter: 'Scripter<br /><span class="small" style="font-weight: normal;">(Scripter)</span>'
  }
  const enabledElement = {
    true: '<div class="serviceLabel"><span class="mr-2 fas fa-circle" style="color: #00cc33;"></span><span style="opacity: 0.4;">Enabled</span></div>',
    false: '<div class="serviceLabel"><span class="mr-2 fas fa-circle text-secondary"></span><span style="opacity: 0.4;">Disabled</span></div>',
  };
  let item = [];
  let itemStart = '<div id="serviceListItem' + index + '" class="serviceListItem draggableService bg-dark" data-id="' + serviceItem.id + '">'
  let itemRemoveBtn = '<div class="removeService" data-toggle="modal" data-target="#removeServiceModal"><span class="fas fa-times-circle" aria-hidden="true" /></div>';
  let itemEnabled = enabledElement[serviceItem.enabled === 1];
  let itemName = '<div class="serviceName">' + nameHelper(serviceItem, serviceName[serviceItem.name]) + '</div>';
  let itemEnd = '</div>';

  item.push(itemStart);
  item.push(itemRemoveBtn);
  item.push(itemName);
  item.push(itemEnabled);
  item.push(itemEnd);

  return item.join("");
}

function createServiceList(serviceArr) {
  let list = [];

  serviceArr.forEach((item, index) => {
    let htmlItemOutput = createServiceCardHTML(item, index);
    list.push(htmlItemOutput)
  });
  let addCard = createServiceAddHtml();
  list.push(addCard);

  let serviceListJqueryObject = $('#serviceList');
  serviceListJqueryObject.html(list.join(""));

  // Add callbacks
  $('.serviceListItem').click(serviceItemClickedCallback);
  // Sortable .serviceListItem
  if ($(window).width() > 769) { // Disable on small devices
    Sortable.create(serviceListJqueryObject[0], {
      handle: '.draggableService',
      filter: '#serviceListItemAdd',
      preventOnFilter: true,
      draggable: ".draggableService",
      dataIdAttr: 'data-id',
      animation: 150,
      ghostClass: 'draggableElementShadow',
      store: {
        set: function (sortable) {
          saveNewOrderOfServices(sortable.toArray());
        },
      }
    });
  }
  $('.serviceListItemActive').removeClass("serviceListItemActive")
  $('#serviceListItem' + activeItem).addClass("serviceListItemActive")
}

function setServiceItemActive(item) {
  activeItem = item;
  $('.serviceListItemActive').removeClass("serviceListItemActive")
  $('#serviceListItem' + item).addClass("serviceListItemActive")

  // Show no service label
  if (!serviceArrTmp.length)
  {
    $("#serviceContent").html('<div class="noServiceAlert"><span class="h4">No service created.</span></div>');
    return;
  }
  else if (item > serviceArrTmp.length - 1)
  {
    setServiceItemActive(serviceArrTmp.length - 1)
    return;
  }

  // Load form for service
  hideImageFromCamera('imagePreviewCamera');
  $("#serviceContent").html('');
  var service = "";
  if (serviceArrTmp[item].name == 'accuweather')
    service = "accuweather/accuweather.html";
  else if (serviceArrTmp[item].name == 'infoticker')
    service = "infoticker/infoticker.html";
  else if (serviceArrTmp[item].name == 'custom' || serviceArrTmp[item].name == 'images') // 'custom' service renamed to 'images'
    service = "custom/custom.html";
  else if (serviceArrTmp[item].name == 'ptzCompass')
    service = "compass/compass.html";
  else if (serviceArrTmp[item].name == 'ptz')
    service = "ptz/ptz.html";
  else if (serviceArrTmp[item].name == 'sportzcast')
    service = "sportzcast/sportzcast.html";
  else if (serviceArrTmp[item].name == 'pip')
    service = "pip/pip.html";
  else if (serviceArrTmp[item].name == 'scripter')
    service = "scripter/scripter.html";
  else if (serviceArrTmp[item].name == 'textAndBackground' || serviceArrTmp[item].name == 'customGraphics') // 'textAndBackground' service renamed to 'customGraphics'
    service = "customGraphics/customGraphics.html";


  if (service.length == 0) {
    console.error('Unknown service: ' + serviceArrTmp[item].name);
    return;
  }

  // Unregister events from controls of form
  $('.form-control').unbind();
  $('.form-control-inline').unbind();
  // Unregistrer events from unexisting items of DOM (draggable lib)
  if (typeof interact !== 'undefined') {
    interact('.boxIn.active').unset();
  }
  // Unregister events from fileManager for specific service
  $('body').off('hide.bs.modal');

  $("#serviceContent").load(service, function() {
    $('[data-toggle="tooltip"]').tooltip({ 'delay': { show: 1000, hide: 0 }});
  });
}

function saveNewOrderOfServices(order) {
  let newServiceArrTmp = [];
  order.forEach(function(item, i) {
    serviceArrTmp.forEach(function(service) {
      if (service.id === parseInt(item)) {
        newServiceArrTmp.push(service);
      }
    });
  });
  serviceArrTmp = [];
  serviceArrTmp = newServiceArrTmp;
  saveBtnCallback(true, newServiceArrTmp);
}

function serviceItemClickedCallback(event) {
  var targ = event.target || event.srcElement;
  var item = $(targ).closest('.serviceListItem');

  var itemID = item[0].id.substring(15);
  if (itemID == 'Add')
    return;
  else if ($(targ).closest('div').hasClass('removeService')) {
    itemForRemove = itemID;
    return;
  }

  setServiceItemActive(itemID);
}

function serviceAddClickedCallback(event) {
  var targ = event.target || event.srcElement;
  var item = $(targ).closest('.addServiceBtn');


  if (item.prop('name') == "optionsRadioAccuWeather") {
    addDefaultService('accuweather');
  } else if (item.prop('name') == "optionsRadioCarousel") {
    addDefaultService('images');
  } else if (item.prop('name') == "optionsRadioPtzCompass") {
    addDefaultService('ptzCompass');
  } else if (item.prop('name') == "optionsRadioInfoTicker") {
    addDefaultService('infoticker');
  } else if (item.prop('name') == "optionsRadioPtz") {
    addDefaultService('ptz');
  } else if (item.prop('name') == "optionsRadioInstant") {
    addDefaultService('instant');
  } else if (item.prop('name') == "optionsRadioSportzcast") {
    addDefaultService('sportzcast');
  } else if (item.prop('name') == "optionsRadioPip") {
    addDefaultService('pip');
  } else if (item.prop('name') == "optionsRadioTextAndBackground"){
    addDefaultService('customGraphics')
  }

  createServiceList(serviceArrTmp);
  setServiceItemActive(serviceArrTmp.length - 1);
}

function addDefaultService(serviceName) {

  //rules for add new service
  rulesForNewService();

  if(serviceName === 'infoticker') {
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "infoticker",
      "customName": "",
      "showClock": 1,
      "clockType": "24h",
      "textColor": "0,0,0,1",
      "bgColor": "255,255,255,1",
      "weatherLocation": "", // location key
      "weatherLocationName": "", // title from location api
      "weatherLang": "en-us",
      "weatherUnits": "Metric",
      "numberOfLines": 1,
      "switchingTime": 5,
      "crawlLeft": false,
      "crawlSpeed": 1.0,
      "coordSystem": "bottom", // bottom|top values
      "pos_y": 0.05, // in percentage
      "width": 1920,
      "height": 1080,
      "font": "classic",
      "fontSize": 40,
      "sourceType": "", // text|url
      "source": ""
    });
  } else if(serviceName === 'accuweather'){
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "accuweather",
      "location": "", // location key
      "locationName": "", // title from location api
      "title": "", // title from user
      "bgStartColor": "0,0,0,0.75",
      "bgEndColor": "0,0,0,0.75",
      "clockType": "24h", // 12h|24h format
      "lang": "en-us",
      "units": "Metric",
      "pos_x": 10,
      "pos_y": 10,
      "width": 1920,
      "height": 1080,
      "coordSystem": "top_right",
      "layout": "0",
      "scale": 1.0
    });
  } else if(serviceName === 'ptzCompass'){
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "ptzCompass",
      "pos_x": 10,
      "pos_y": 10,
      "width": 1920,
      "height": 1080,
      "coordSystem": "top_left",
      "scale": 1.0,
      "type": "compass", // compass|image
      "image": "", // file:///usr/local/packages/camoverlay/localdata/user_images/vodnik-1.png
      "northPan": 0.0, // pan in degrees
      "cameraPosX": 0, // used only if type is 'map' or 'image'
      "cameraPosY": 0,
      "colorScheme": "black" // black|white|orange
    });
  } else if (serviceName === 'images') { // old name was 'custom'
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "images",
      "width": 1920,
      "height": 1080,
      "overlayList": []
    });
  } else if (serviceName === 'ptz') {
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "ptz",
      "width": 1920,
      "height": 1080,
      "ptz_positions": {}
    });
  } else if (serviceName === 'instant') {
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "instant",
      "width": 1920,
      "height": 1080,
      "scale": 1.0,
      "overlayList": []
    });
  } else if (serviceName === 'sportzcast') {
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "sportzcast",
      "xml_url": ""
    });
  } else if (serviceName === 'pip') {
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "pip",
      "customName": "",
      "width": 1920,
      "height": 1080,
      "coordSystem": "top_left",
      "pos_x": 10,
      "pos_y": 10,
      "fps": 1,
      "compression": 50,
      "source_type": "axis_camera", // axis_camera|mjpeg_url
      "mjpeg_url": "",
      "mjpeg_width": 320,
      "mjpeg_height": 240,
      "camera_ip": "",
      "camera_port": 80,
      "camera_user": "root",
      "camera_pass": "",
      "camera_width": 320,
      "camera_height": 240,
      "camera_view_area": "1"
    });
  } else if (serviceName === 'customGraphics'){ // old name was 'textAndBackground'
    serviceArrTmp.push({
      "id": getUniqueID(),
      "enabled": 1,
      "automationType": "manual",
      "camera": 0,
      "schedule": "",
      "name": "customGraphics",
      "customName": "",
      "pos_x": 10,
      "pos_y": 10,
      "width": 1920,
      "height": 1080,
      "coordSystem": "top_left",
      "clockFormat": "24h",
      "background": "custom", // custom|image
      "image": "", // /usr/local/packages/camoverlay/localdata/user_images/vodnik-1.png
      "customAreaColor": "0,0,0,0.8",
      "customAreaWidth": 300,
      "customAreaHeight": 200,
      "customAreaCorners": "rounded",
      "mappingZones": [{
        "name": "field1",
        "pos_x": 10,
        "pos_y": 10,
        "text": [{
          "source": "",
          "active": true
        }],
        "wrapText": false,
        "textLines": 1,
        "textWidth": 200,
        "textAlign": "A_LEFT",
        "textColor": "255,0,0,1.0",
        "font": "classic",
        "fontSize": 30,
        "switchingTime": 0
      }]
    });
  }
}

function rulesForNewService(){
  if (!isMips()) {
    if (enabledServicesCount() === 2 || enabledServicesCount() === 3) {
      showMessage('warning', 'Three or four active services in one Video Channel in CamOverlay App could be in conflict with Axis overlay.', false, 'threeAndFourServices');
    }
  }
}

function getUniqueID() {
  // Get max id from all services
  var maxID = 0;
  for (var i = 0; i < serviceArrTmp.length; i++) {
    if (serviceArrTmp[i].id != undefined && serviceArrTmp[i].id > maxID)
      maxID = serviceArrTmp[i].id;
  }
  return maxID + 1;
}

async function serviceRemoveClickedCallback() {
  serviceArr = await getServices();
  serviceArrTmp.splice(itemForRemove, 1);
  serviceArr.splice(itemForRemove,1);
  var backupActiveItem = activeItem;
  if (backupActiveItem > itemForRemove)
    backupActiveItem--;
  createServiceList(serviceArrTmp);
  setServiceItemActive(backupActiveItem);
  saveBtnCallback(true, serviceArr);
}

function saveBtnCallback(silence, services) {
  if(!services) {
    serviceArr = JSON.parse(JSON.stringify(serviceArrTmp));
  } else {
    serviceArr = JSON.parse(JSON.stringify(services));
  }

  // Save params to camera
  var servicesObj = {"services" : serviceArr};
  var servicesStr = JSON.stringify(servicesObj);
  setServices(servicesStr).then(function() {
    hideSaveWarning();
    if (silence !== true)
      showMessage('success', 'Parameters have been successfully updated.', false, 'successParameters');
  }, function() {
    if (silence !== true)
      showMessage('danger', 'Parameters setting error.', false, 'errorParameters');
  });

  let enabledServices = getEnabledServices();

  // When one service is turned on, set global param 'enabled' to 1
  if (enabledServices.length > 0) {
    setParameter('CamOverlay','Enabled', '1');
  }

  // Send statistic data
  var data = new FormData();
  data.append('services', enabledServices);
  jQuery.ajax({
    method: 'POST',
    url: serverUrl + '/index.php?p=CamOverlay&a=updateServices&serial_number=' + serialNumber,
    data: data,
    cache: false,
    contentType: false,
    processData: false
  });

}

function discardBtnCallback() {
  serviceArrTmp = JSON.parse(JSON.stringify(serviceArr));
  setServiceItemActive(activeItem);
  createServiceList(serviceArrTmp);
}

/**
 * Create url for image preview from camera
 * @param {string} elementId - set att src to this element
 * @param {Object} resolution - contains width and height prop
 * @param {?number} videoChannel - number of channel of camera
 */
function showImageFromCamera(elementId, resolution, videoChannel) {
  let previewUrl = new URL(cameraPreviewUrlPath, window.location.origin);
  let urlParams = new URLSearchParams(previewUrl.search);
  urlParams.set('fps', 2);
  if (resolution !== undefined && Object.keys(resolution).length !== 0 ) {
    urlParams.set('resolution', resolution.width + 'x' + resolution.height);
  }
  if (videoChannel !== undefined && videoChannel !== null && videoChannel > -1) {
    urlParams.set('camera', ++videoChannel);
  }
  if (params['image.i'  + videoChannel + '.source'] === 'quad') {
    urlParams.set('camera', 'quad');
  }
  urlParams.set('t', Date.now());
  previewUrl.search = urlParams.toString();
  $('#' + elementId).attr("src", previewUrl);
}

function hideImageFromCamera(elementId) {
  $('#' + elementId).attr("src", "");
}

function initTooltips() {
  var tooltips = getFormTooltips();
  insertTooltip($('select[id*="Enabled"]').closest('.input-group'), tooltips['enabled']);
  insertTooltip($('select[id*="VideoChannel"]').closest('.input-group'), tooltips['videochannel']);
  $('[title]').not('.no-tooltip').tooltip();
}

function insertTooltip(element, tooltip){
  element.attr('title', tooltip);
}

function getFormTooltips(){
  /* different tooltips for architecture of camera */
  if(isMips()) {
    return { // artpec 5 and mips
      enabled: 'Switch ON if you want to show graphics overlay for this service. You can run up to 2 services at the same time.',
      videochannel: ''
    }
  } else {
    return { // artpec 6 and higher
      enabled: 'Switch ON if you want to show graphics overlay for this service. You can run up to 4 services at the same time in one Video Channel.',
      videochannel: 'If you want to stream independent graphics for a particular stream in the CamStreamer App, you\'ll have to create an independent View Area. Before selecting a new service in the CamStreamer App, select a particular Video Channel in the drop-down box at the top of the page.'
    }
  }
}

function isArmv7hf() {
 return cameraArchitecture === 'armv7hf';
}

function isMips() {
  return cameraArchitecture === 'mips';
}

function enabledServicesPerVideoChannel(currentVideoChannel){
  var count = 0;
  for (var i = 0; i < serviceArrTmp.length; i++){
    if (serviceArrTmp[i].enabled === 1) {
      if(currentVideoChannel === serviceArrTmp[i].camera) count++;
    }
  }
  return count;
}

function enabledServicesCount() {
  var count = 0;
  for (var i = 0; i < serviceArrTmp.length; i++) {
    if (serviceArrTmp[i].enabled === 1)
      count++;
  }
  return count;
}

function getEnabledServices() {
  var serviceNames = '';
  for (var i = 0; i < serviceArr.length; i++) {
    if (serviceArr[i].enabled === 1) {
      if (serviceNames.length)
        serviceNames += ',';
      serviceNames += serviceArr[i].name;
    }
  }
  return serviceNames;
}

function createScheduleBtn() {
  var btnDesc = '<span class="fa fa-calendar-alt mr-1"></span> Set day and time'
  if (serviceArrTmp[activeItem].schedule != undefined && serviceArrTmp[activeItem].schedule.length)
    btnDesc += ' <span class="fa fa-check-circle mr-1"></span>'
  $('#scheduleBtn').html(btnDesc);
  $('#scheduleBtn').unbind().click(scheduleClickedCallback);
  $('#fromDay').unbind().click(scheduleFromDayChangeCallback);
  $('#fromDay').change(scheduleFromDayChangeCallback);
}

function scheduleClickedCallback() {
  $("#scheduleModal").modal({show: true});
  scheduleTmpArr = [];
  if (serviceArrTmp[activeItem].schedule != undefined)
    scheduleTmpArr = serviceArrTmp[activeItem].schedule.split('|');
  createScheduleTable(scheduleTmpArr);
}

function createScheduleTable(scheduleArr) {
  var table = '<tbody>';
  if (scheduleArr.length) {
    for (var i = 0; i < scheduleArr.length; i++) {
      var interval = scheduleArr[i].trim();
      if (interval.length == 0) {
        scheduleTmpArr = [];
        createScheduleTable(scheduleTmpArr);
        return;
      }
      var intervalArr = interval.split('..');
      table += '<tr class="schedule" data-schedule="' + interval + '">';
      table += '<td>' + intToDay(parseInt(intervalArr[0][0])) + ' ' + intervalArr[0].substring(2) + '</td>';
      table += '<td> - </td>';
      table += '<td>' + intToDay(parseInt(intervalArr[1][0])) + ' ' + intervalArr[1].substring(2) + '</td>';
      table += '<td><button id="removeSchedule' + i + '" class="btn btn-sm btn-primary removeSchedule">Remove</button></td>';
      table += '</tr>';
    }
  }
  else
    table += '<tr class="schedule"><td colspan="4">No schedule set.</td></tr>';

  table += '</tbody>';

  $('#scheduleList').html(table);
  $('.removeSchedule').click(removeScheduleClickedCallback);
}

function scheduleFromDayChangeCallback() {
  $('#toDay').val($('#fromDay').val());
}

function scheduleAddClickedCallback() {
  var fromDay = $('#fromDay')[0].selectedIndex + 1;
  var fromTime = $('#fromTime').val();
  var toDay = $('#toDay')[0].selectedIndex + 1;
  var toTime = $('#toTime').val();
  scheduleTmpArr.push(fromDay + ' ' + validateTime(fromTime, '00:00:00') + '..' + toDay + ' ' + validateTime(toTime, '23:59:59'));
  createScheduleTable(scheduleTmpArr);
  return false; // Prevent form submission
}

function validateTime(inputTime, defaultVal) {
  var tArr = inputTime.split(':');
  var tArrValidated = [];
  for (var i = 0; i < 3; i++) {
    if (i < tArr.length) {
      var num = parseInt(tArr[i]);
      if (!Number.isNaN(num)) {
        tArrValidated.push(('0' + num).slice(-2));
      } else {
        console.log('Time conversion error: ' + tArr + ', default value is used: ' + defaultVal);
        tArrValidated = defaultVal.split(':');
        break;
      }
    } else {
      tArrValidated.push('00');
    }
  }
  return tArrValidated.join(':');
}

function removeScheduleClickedCallback(event) {
  var targ = event.target || event.srcElement;
  var itemPos = targ.id.substring(14);
  scheduleTmpArr.splice(itemPos, 1);
  createScheduleTable(scheduleTmpArr);
}

function scheduleSaveClickedCallback() {
  var schedule = '';
  for (var i = 0; i < scheduleTmpArr.length; i++) {
    schedule += scheduleTmpArr[i];
    if (i < scheduleTmpArr.length - 1)
      schedule += '|';
  }
  serviceArrTmp[activeItem].schedule = schedule;
  showSaveWarning();
  createScheduleBtn();
}

function intToDay(dayNum) {
  switch (dayNum) {
    case 1:
      return 'Mon';
    case 2:
      return 'Tue';
    case 3:
      return 'Wed';
    case 4:
      return 'Thu';
    case 5:
      return 'Fri';
    case 6:
      return 'Sat';
    case 7:
      return 'Sun';
    default:
      return '';
  }
}

function showTrialFormClicked() {
  $('#panelLicenseCode')[0].className = 'card card-default';
  $('#panelTrial')[0].className = 'card card-success';
  $("#licenseContent").load("activateTrialForm.html", function() {
    $("#licenseFormActivateTrial").click(activateTrialClicked);
  });
}

function showLicenseFormClicked() {
  $('#panelLicenseCode')[0].className = 'card card-success';
  $('#panelTrial')[0].className = 'card card-default';
  $("#licenseContent").load("activateForm.html", function() {
    $("#licenseFormActivate").click(activateClicked);
  });
}

function activateTrialClicked() {
  var email = encodeURIComponent($('#email').val());
  if (email.length == 0) {
    $('#emailGroup').addClass('has-error');
    return false;
  }

  var dataJson = {
    'a': 'registerlicense',
    'p': 'camoverlay',
    'serial_number': serialNumber,
    'license_code': 'TRIAL',
    'email': email
  }

  $.ajax({
    method: "GET",
    url: serverUrl + "/index.php",
    data: dataJson,

    success: function (response) {
      if (response.code == 200) {
        showLicenseModalMessage('');
        window.location.reload(); // Reload page
      } else {
        showLicenseModalMessage(response.message);
      }
    }
  });

  return false;
}

function activateClicked() {
  var license = encodeURIComponent($('#licenseCode').val());
  if (license.length == 0) {
    $('#licenseCodeGroup').addClass('has-error');
    return false;
  }

  license = license.replace(/^\s+|\s+$/gm,'');
  license = license.replace(/%20/g, "");

  var dataJson = {
    'a': 'registerlicense',
    'p': 'camoverlay',
    'serial_number': serialNumber,
    'license_code': license
  }

  $.ajax({
    method: "GET",
    url: serverUrl + "/index.php",
    data: dataJson,

    success: function (response) {
      if (response.code == 200) {
        showLicenseModalMessage('');
        window.location.reload(); // Reload page
      } else {
        showLicenseModalMessage(response.message);
      }
    }
  });

  return false;
}

function showLicenseInfoCallback() {
  var licenseInfo = 'No license info.';
  if (licenseActivated != -1 && licenseExpire != -1 && licenseCode != -1) {
    var d = new Date(licenseActivated * 1000);
    licenseInfo = '<dl class="dl-horizontal">';
    licenseInfo += '<dt>License code</dt><dd>' + licenseCode + '</dd>';
    licenseInfo += '<dt>License type</dt><dd>' + (licenseExpire == 0 ? 'Unlimited' : 'Trial') + '</dd>';
    licenseInfo += '<dt>Activated</dt><dd>' + d.getDate() + '.' + (d.getMonth() + 1) + '.' + d.getFullYear() + '</dd>';
    licenseInfo += '</dl>';
  }

  $('#licenseInfoBody').html(licenseInfo);
  $('#licenseInfoModal').modal('show');
}

function showExpiredLicense(){

  $('.content').hide();
  $('#imageManagerBtn').hide();
  $('.expired_license').show();
}

function showLicenseModal(disableTrial, allowExit) {
  if (allowExit) {
    $('#licenseModal').attr('data-backdrop', true);
    $('#licenseModalCloseBtn').show();
  } else {
    $('#licenseModal').attr('data-backdrop', 'static');
    $('#licenseModalCloseBtn').hide();
  }

  $('#licenseModal').modal('show');
  $("#showLicenseForm").unbind().click(showLicenseFormClicked);

  if (disableTrial) {
    $("#showTrialForm").unbind();
    $("#licensePanelTrial").addClass('disabled');
  } else {
    $("#showTrialForm").unbind().click(showTrialFormClicked);
    $("#licensePanelTrial").removeClass('disabled');
  }
}

function showLicenseModalMessage(msg) {
  if (msg.length == 0) {
    $('#licenseModalMessages').html('');
    return;
  }

  var message = '<div class="alert alert-dismissible alert-danger">';
  message += '<button type="button" class="close" data-dismiss="alert">&times;</button>'
  message += msg + '</div>';
  $('#licenseModalMessages').html(message);
}

function getPreviewImage(request) {
  previewRequests.push(request);
  if (previewRequests.length == 1) {
    var activeItemBackup = activeItem;
    $.get(request, function(response) {
      if (activeItem == activeItemBackup && serviceArrTmp[activeItem].enabled) {
        var img = '<img id="imgPreview" style="display: none" src="data:image/png;base64,' + response.data + '">';
        $("#imagePreviews").html(img);
        setPreviewPos(response.pos_x, response.pos_y, response.width, response.height, response.stream_width, response.stream_height);
      }
      previewRequests.shift();
      var newReq = previewRequests.pop();
      previewRequests = [];
      if (newReq != undefined)
        getPreviewImage(newReq);
    });
  }
}

function showControlRoomCallback() {
  $('body').addClass('modal-overlay');
  hideImageFromCamera('imagePreviewCamera');
  $('#controlRoomContainer').load("controlRoom.html", function() {}).show();
}

function hideControlRoomCallback() {
  $('body').removeClass('modal-overlay');
  $('#controlRoomContainer').hide().html('');
  setServiceItemActive(activeItem); // reload whole service, because of changed inputs
}

async function showImageManagerCallback() {
  await imageManager.init();
  imageManager.open();
}

async function showFontManagerCallback() {
  await fontManager.init();
  fontManager.open();
}

function startAcapApp() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/axis-cgi/applications/control.cgi?package=camoverlay&action=start', function(response) {
      response = response.trim().toLowerCase();

      var errorCode = ''
      var found = response.search('error:');
      if (found != -1)
        errorCode = response.substring(7);

      if (response == 'ok') {
        setTimeout(resolve, 2000);  // Wait for acap start
      } else if (errorCode == 6) {
        resolve();
      } else {
        showMessage('danger', 'Could not start camoverlay package, error: ' + errorCode + '.');
        resolve();
      }
    });
  });
  return promise;
}

function showMessage(type, html, static, msgName) {
  var header = 'Info:';
  if (type == 'success')
    header = 'Success:';
  else if (type == 'warning')
    header = 'Warning!';
  else if (type == 'danger')
    header = 'Error!';

  var id = alertID++;
  var exists = 0;
  var targetDiv = (static === false || static === undefined) ? '#messages_fluid' : '#messages';

  if ($(targetDiv).find('.' + msgName).length > 0) exists = 1;
  if (exists === 0 || msgName === undefined) {
    if (msgName === undefined) msgName = "";
    var message = '<div id="alert' + id + '" class="alert alert-dismissible alert-' + type + ' ' + msgName + '">';
    if(static !== false) { message += '<button type="button" class="close" data-dismiss="alert">&times;</button>'; }
    message += '<strong class="mr-2">' + header + '</strong>';
    message += html + '</div>';
    $(targetDiv).append(message);
  }

  if (static === undefined || static === false) {
    $('#alert' + id).fadeTo(5000, 500).slideUp(500, function(){
      $('#alert' + id).alert('close');
    });
  }
}

function validateLicense(serverUrl, serialNumber, cameraModel, cameraFirmware, appVersion) {
  var dataJson = {
    'a': 'checklicense',
    'p': 'camoverlay',
    'serial_number': serialNumber,
    'model': cameraModel,
    'firmware': cameraFirmware,
    'app_ver': appVersion,
  }

  var promise = new Promise(function(resolve, reject) {
    $.ajax({
      method: "GET",
      url:  serverUrl + "/index.php",
      data: dataJson,

      success: function (response) {
        resolve(response);
      },

      error: function (response) {
        resolve();
      }
    });
  });
  return promise;
}
